function [Xout] = ALMMAnalysisl1(Y,X,Omega,lambda,IT)
% Augmented Lagrangian Multiplier Method for Analysis l1
% The operator Omega is assummed to be Tight Frame
% Y = Noisy Cosparse Signals
% X = Initial (Clean) Cosparse Signal (if unknown, you can use Y as the initial value)
% Omega = Analysis Operator
% lambda = Lagrange Multiplier of the Objective
% IT = Maximum Iteration Number
% Xout = Denoised Signals


% Constrained Analysis Operator Learning for Cosparse Signal Modelling
% Written by Mehrdad Yaghoobi, version 1.0                                    
%     
% Copyright 2013 Mehrdad Yaghoobi, Sangnam Nam, Remi Gribonval and Mike E. Davies
% 
% For all details please refer to README.m
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
                                                                                                                                                      


gamma = lambda; % Augmented LMM coefficient
Theta = zeros(size(Omega,1),size(X,2));   
Z = Omega*X; % Parameter Splitting
i = 1;
while (i <= 5) || (norm(Z-Omega*X,'fro') >= .01) && (i <= IT)
    X = (lambda*Y + gamma*Omega'*(Z-Theta))/(lambda+gamma);
    Z = softthreshold(Omega*X+Theta,1/(gamma)); % Soft-thresholding
    Theta = Theta - (Z-Omega*X); % Updating the Lagrange parameter of ALMM
    i = i+1;
end
Xout = X;