function [] = NoisyAOLFaceRandLine(M,N,L,lambda)
% Noise Aware Analysis Operator Learning with a Line Search
% M = Signal Dimensiona (8 by 8 image patches)
% N = Analysis Space Dimension
% L = Number of Training Samples
% lambda = Lagrange Multiplier in the Objective Function

% Constrained Analysis Operator Learning for Cosparse Signal Modelling
% Written by Mehrdad Yaghoobi, version 1.0                                    
%     
% Copyright 2013 Mehrdad Yaghoobi, Sangnam Nam, Remi Gribonval and Mike E. Davies
% 
% For all details please refer to README.m
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose. 

load(['ParamFaceYale',num2str(M),'X',num2str(N),'X',num2str(L),'.mat'])

Omega = D';
[M,L] = size(Y);    

% Removing DC component

Y = double(Y) - ones(M,1)*mean(Y);

% Initialisation

X = Y; % Initialising X
alpha = 10^-7; % Initial Sub-gradient Step Size
IN = 10000; % Number of inner-loop iteration
c = f(Omega,X,Y,lambda);
cost(1) = c;

% Alternating minimisation of the operator update and the cosparse
% coefficients update.

for k = 1:10,
    k          
        
    % Projected Sub Gradient With Line Search
    
    [Omega,ilcost] = SubGPAOLwLS(X,Omega,IN,alpha);
    
    % Augmented Lagrangian MM for Cosparse Signal Update
    
    X = ALMMAnalysisl1(Y,X,Omega,lambda,1000);
    
    cost(k+1) = f(Omega,X,Y,lambda);
end

save(['FaceNoisyAOLLine',num2str(M),'X',num2str(N),'X',num2str(L),'X',num2str(lambda),'.mat'],'Omega','Y','X','cost')

function c = f(Omega,X,Y,lambda)
c = sum(sum(abs(Omega*X)))+ .5*lambda*norm(X-Y,'fro')^2;
